@@ -9,7 +9,7 @@
  * \wd \taz \greg \mysid
  * \date 1996-2001
  *
- * $Id: akill.c,v 1.1.1.1 2003/07/04 02:50:53 Mysid Exp $
+ * $Id: akill.c,v 1.16 2005/01/07 05:34:17 echostar Exp $
  */
 
 /*
@@ -50,6 +50,22 @@
 #include "email.h"
 #include "log.h"
 #include "sipc.h"
+#include "timestr.h"
+
+/*******************************************************************/
+#define NUM_AKTYPE_INDICES 4
+
+struct _akill_mail_exclude {
+	char* nick;
+	int count_removes[NUM_AKTYPE_INDICES];
+	int count_adds[NUM_AKTYPE_INDICES];
+
+	LIST_ENTRY(_akill_mail_exclude) dm_lst;
+};
+
+typedef struct _akill_mail_exclude SetterExclude;
+
+static LIST_HEAD(, _akill_mail_exclude)  mailExcludedSetters;
 
 /*******************************************************************/
 
@@ -118,30 +134,95 @@ int IpcConnectType::sendAkills(int akillType, const char *searchText)
 	return x;
 }
 
+/**
+ * \brief Excluded the given nick from akill mailings to ops@ and kline@
+ * \param nick -- The setter to be excluded, non-null zero terminated char array
+ */
+void makeSetterExcludedFromKlineMails(const char* nick)
+{
+	SetterExclude* se;
+	int i;
+	
+	if (nick == 0 || !*nick)
+		return;
+
+	// Never list one twice
+	for(se = LIST_FIRST(&mailExcludedSetters); se; se = LIST_NEXT(se, dm_lst))
+	{
+		if (se->nick && !str_cmp(se->nick, nick))
+			return;
+	}
+
+	se = (SetterExclude*)oalloc(sizeof(SetterExclude));
+	LIST_ENTRY_INIT(se, dm_lst);
+	se->nick = (char *)oalloc(strlen(nick)+1);
+	strcpy(se->nick, nick);
+
+	for(i = 0; i < NUM_AKTYPE_INDICES; i++) {
+		se->count_adds[i] = 0;
+		se->count_removes[i] = 0;
+	}
+
+	LIST_INSERT_HEAD(&mailExcludedSetters, se, dm_lst);
+}
+
 /**
  * \brief Handles an AutoKill/AutoHurt/Ignore/... list request
  * \param from Oper requesting list
  * \param type Type of list requested (ie: #A_AKILL)
+ * \param search Search string
+ * \param method Search method
  * \pre From points to a valid NUL-terminated character array bearing
  *      the nickname of an online IRC oper.  Type is one of the akill
  *      type constants (ex: #A_AKILL, #A_IGNORE, #A_AHURT);
  */
-void listAkills(char *from, char type)
+void listAkills(char *from, char type, char *search, ak_search_method method)
 {
 	struct akill *ak;
 	char mask[NICKLEN + USERLEN + HOSTLEN + 3];
-	char length[16];
+	char length[16] = {};
 	struct tm *t;
+	int matched = 0, total = 0;
+	MaskData *umask = NULL;
 
 	if (!firstBanItem) {
 		sSend(":%s NOTICE %s :There are no autokills/ignores.", OperServ,
 			  from);
 		return;
 	}
-	sSend(":%s NOTICE %s :%-14s %-8s %-20s %-10s %s", OperServ, from,
-		  "SetTime", "Length", "Mask", "Set By", "Reason");
+
+	if (method != SEARCH_NONE && (!search || strlen(search) == 0)) {
+		sSend(":%s NOTICE %s :Bad list parameters", OperServ, from);
+		return;
+	}
+
+	if (method == SEARCH_UMASK) {
+		umask = make_mask();
+
+		if (split_userhost(search, umask) < 0) {
+			free_mask(umask);
+
+			sSend(":%s NOTICE %s :Unable to read mask.", NickServ,
+			      from);
+			return;
+		}
+	}
+
 	for (ak = firstBanItem; ak; ak = ak->next) {
-		if (ak->type == type || ak->type == 0) {
+		if ((ak->type == type || ak->type == 0) &&
+		    ((method == SEARCH_UMASK &&
+		      match(umask->nick, ak->nick) == 0 &&
+		      match(umask->user, ak->user) == 0 &&
+		      match(umask->host, ak->host) == 0) ||
+		     (method == SEARCH_REASON &&
+		      match(search, ak->reason) == 0) ||
+		     (method == 0))) {
+
+			if (matched == 0)
+				sSend(":%s NOTICE %s :%-14s %-8s %-20s %-10s %s",
+				      OperServ, from, "SetTime", "Length",
+				      "Mask", "Set By", "Reason");
+
 			strcpy(mask, ak->nick);
 			strcat(mask, "!");
 			strcat(mask, ak->user);
@@ -150,9 +231,22 @@ void listAkills(char *from, char type)
 
 			t = localtime(&(ak->set));
 
-			if (ak->unset)
-				sprintf(length, "%luh",
-						(long)(ak->unset - ak->set) / 3600);
+			if (ak->unset) 
+			{
+				TimeLengthString tls(ak->unset - ak->set);
+
+				ASSERT((ak->unset - ak->set) >= 0);
+
+				if (tls.isValid() == false) 
+				{
+					sprintf(length, "%luh",
+							(long)(ak->unset - ak->set) / 3600);
+				}
+				else
+				{
+					tls.asString(length, sizeof(length) - 1, true, false, false);
+				}
+			}
 			else
 				strcpy(length, "forever");
 
@@ -161,9 +255,27 @@ void listAkills(char *from, char type)
 				 OperServ, from, t->tm_mon + 1, t->tm_mday,
 				 (t->tm_year % 100), t->tm_hour, t->tm_min, length, mask,
 				 ak->setby, ak->reason);
+			matched++;
+		}
+
+		total++;
+	}
+
+	if (method != SEARCH_NONE) {
+		if (matched > 0) {
+			sSend(":%s NOTICE %s :Matched %d/%d autokill/hurts",
+			      OperServ, from, matched, total);
+		} else {
+			sSend(":%s NOTICE %s :No autokills/hurts match",
+			      OperServ, from);
 		}
 	}
-	sSend(":%s NOTICE %s :End of list.", OperServ, from);
+
+	if (method == SEARCH_UMASK)
+		free_mask(umask);
+
+	if (method == SEARCH_NONE || matched > 0)
+		sSend(":%s NOTICE %s :End of list.", OperServ, from);
 }
 
 /**
@@ -171,34 +283,29 @@ void listAkills(char *from, char type)
  * \param nick The user's nickname
  * \param user The users's username
  * \param host The user's hostname
+ * \param type The type of ban (A_AKILL, A_AHURT, A_IGNORE), can be 
+ *	bitwise or'ed together to check for multiple types
  * \pre Nick, user, and host each point to separate, valid
  *      NUL-terminated character arrays.  Nick is a valid
  *      IRC nickname, user is a valid IRC username, and host
- *      is a valid IRC hostname.
+ *      is a valid IRC hostname. Type is one of the above ban types,
+ *	or a combination of them bitwise or'ed together.
  */
-int isAKilled(char *nick, char *user, char *host)
+struct akill* getBanInfo(char *nick, char *user, char *host, char type)
 {
+#ifndef ENABLE_AHURT
+	if(type == A_AHURT)
+	    return NULL;
+#endif
+	
 	struct akill *ak;
 
 	for (ak = firstBanItem; ak; ak = ak->next) {
 		if (!match(ak->nick, nick) && !match(ak->user, user)
-			&& !match(ak->host, host) && (ak->type == A_AKILL))
-			return 1;
-	}
-	return 0;
-}
-
-
-struct akill* getAkill(char *nick, char *user, char *host)
-{
-	struct akill *ak;
-
-	for (ak = firstBanItem; ak; ak = ak->next) {
-		if (!match(ak->nick, nick) && !match(ak->user, user)
-			&& !match(ak->host, host) && (ak->type == A_AKILL))
+			&& !match(ak->host, host) && (ak->type & type))
 			return ak;
 	}
-	return (struct akill*)0;
+	return NULL;
 }
 
 long getAkillId(struct akill* ak)
@@ -206,18 +313,6 @@ long getAkillId(struct akill* ak)
 	return ak->id;
 }
 
-struct akill* getAhurt(char *nick, char *user, char *host)
-{
-	struct akill *ak;
-
-	for (ak = firstBanItem; ak; ak = ak->next) {
-		if (!match(ak->nick, nick) && !match(ak->user, user)
-			&& !match(ak->host, host) && (ak->type == A_AHURT))
-			return ak;
-	}
-	return (struct akill*)0;
-}
-
 char* getAkReason(struct akill *ak)
 {
 	return ak->reason;
@@ -265,12 +360,12 @@ char* applyAkill(char* nick, char* user, char* host, struct akill* ak)
 
 		if (ak->type == A_AKILL)
 			sprintf(buf,
-				"[#%.6x] Autokill %s!%s@%s triggered by user %s!%s@%s at %s\n",
+				"[#%.6x] Autokill %s!%s@%s enforced on user %s!%s@%s at %s\n",
 				ak->id, ak->nick, ak->user, ak->host, nick, user,
 				host, ctime(&CTime));
 		else if (ak->type == A_AHURT)
 			sprintf(buf,
-				"[#%.6x] Autohurt %s!%s@%s triggered by user %s!%s@%s at %s\n",
+				"[#%.6x] Autohurt %s!%s@%s enforced on user %s!%s@%s at %s\n",
 				ak->id, ak->nick, ak->user, ak->host, nick, user,
 				host, ctime(&CTime));
 #ifdef AKILLMAILTO
@@ -290,53 +385,6 @@ char* applyAkill(char* nick, char* user, char* host, struct akill* ak)
 	return NULL;
 }
 
-/**
- * \brief Returns #TRUE if a user is autohurt
- * \param nick The user's nickname
- * \param user The users's username
- * \param host The user's hostname
- * \pre Nick, user, and host each point to separate
- *      NUL-terminated character arrays.  Respectively,
- *      a valid nickname, a valid username, and a valid IRC
- *      hostname.
- */
-int isAHurt(char *nick, char *user, char *host)
-{
-#ifdef ENABLE_AHURT
-	struct akill *ak;
-
-	for (ak = firstBanItem; ak; ak = ak->next) {
-		if (!match(ak->nick, nick) && !match(ak->user, user)
-			&& !match(ak->host, host) && (ak->type == A_AHURT))
-			return 1;
-	}
-#endif
-	return 0;
-}
-
-/**
- * \brief Returns #TRUE if a user is ignored
- * \param nick The user's nickname
- * \param user The users's username
- * \param host The user's hostname
- * \pre Nick, user, and host each point to separate
- *      NUL-terminated character arrays.  Respectively,
- *      a valid nickname, a valid username, and a valid IRC
- *      hostname.
- */
-int isIgnored(char *nick, char *user, char *host)
-{
-	struct akill *ak;
-
-	for (ak = firstBanItem; ak; ak = ak->next) {
-		if (!match(ak->nick, nick) && !match(ak->user, user)
-			&& !match(ak->host, host) && (ak->type == A_IGNORE))
-			return 1;
-	}
-	return 0;
-}
-
-
 /** \brief Apply a new autokill/ban to online users
  *  \param Pointer to an akill item (struct akill) to be applied
  *  \pre   Ak is a valid, listed autokill object.
@@ -397,7 +445,8 @@ void checkAkillAllUsers(struct akill *ak)
 					  ak->reason);
 
 				sprintf(buf,
-						"Autokill %s!%s@%s triggered by %s!%s@%s at %s\n",
+						"[#%.6x] Autokill %s!%s@%s enforced on existing user %s!%s@%s at %s\n",
+						ak->id,
 						ak->nick, ak->user, ak->host, tmpnick->nick,
 						tmpnick->user, tmpnick->host, ctime(&CTime));
 #ifdef AKILLMAILTO
@@ -451,7 +500,7 @@ int addakill(long length, char *mask, char *by, char type, char *reason)
 	struct akill *ak;
 	char *akmask;
 	int i, j;
-	char temp[16];
+	char temp[75] = {};
 
 	ak = (struct akill *)oalloc(sizeof(struct akill));
 	akmask = strdup(mask);
@@ -489,8 +538,14 @@ int addakill(long length, char *mask, char *by, char type, char *reason)
 
 	if (length == 0)
 		strcpy(temp, "forever.");
-	else
-		snprintf(temp, 16, "%d hours.", (int)length / 3600);
+	else {
+		TimeLengthString tls(length);
+
+		if (tls.isValid() == false)
+			snprintf(temp, 16, "%d hours.", (int)length / 3600);
+		else
+			tls.asString(temp, sizeof(temp) - 1, true, true, false);
+	}
 
 	sSend(":%s GLOBOPS :%s set %s for %s for %s [%s]", OperServ, by,
 		  aktype_str(type, 0), mask, temp, reason);
@@ -520,6 +575,30 @@ int addakill(long length, char *mask, char *by, char type, char *reason)
 }*/
 #endif
 
+// Used by queueAkill and kline mailing only
+int akill_flag_toindex(int type_flag)
+{
+	if (type_flag & A_AKILL)
+		return 0;
+	if (type_flag & A_AHURT)
+		return 2;
+	if (type_flag & A_IGNORE)
+		return 1;
+	return 3;
+}
+
+// Used by queueAkill and kline mailing only
+int akill_index_toflag(int index)
+{
+	switch(index)
+	{
+		case 0: return A_AKILL;
+		case 1: return A_IGNORE;
+		case 2: return A_AHURT;
+		default: return A_AKILL;
+	}
+}
+
 
 /**
  * \brief Adds information about an autokill to the body of the kline queue
@@ -537,7 +616,7 @@ void queueakill(char *mask, char *setby, char *length, char *reason,
 				time_t time, int type, int id, int added)
 {
 	static char buf[8192];
-
+	SetterExclude* se, *excluded = 0;
 
 	if (added) {
 		sprintf(buf,
@@ -552,15 +631,41 @@ void queueakill(char *mask, char *setby, char *length, char *reason,
 				ctime(&(time)));
 	}
 
+	for(se = LIST_FIRST(&mailExcludedSetters); se; se = LIST_NEXT(se, dm_lst))
+	{
+		if (se->nick && !str_cmp(se->nick, setby))
+		{
+			excluded = se;
+			break;
+		}	
+	}
+
 #ifdef AKILLMAILTO
 	kline_email.body.add(buf);
 	kline_email_nitems++;
 #endif
 
-#ifdef OPSMAILTO
-	ops_email.body.add(buf);
-	ops_email_nitems++;
-#endif
+#ifdef OPSMAILTO	
+	if (excluded == 0)
+	{
+		ops_email.body.add(buf);
+		ops_email_nitems++;
+	}
+	else {		
+		int x = akill_flag_toindex(type);
+
+		if (x >= 0 && x < NUM_AKTYPE_INDICES) 
+		{
+			if (added) {
+				excluded->count_adds[x]++;
+			}
+			else {
+				excluded->count_removes[x]++;
+			}	
+		}
+		ops_email_nitems++;
+	}
+#endif	
 }
 
 
@@ -618,7 +723,9 @@ const char *aktype_str(int type, int which)
  */
 void timed_akill_queue(char *)
 {
-	char buf1[1024];
+	char buf1[1024], buf2[1024];
+	SetterExclude* se = LIST_FIRST(&mailExcludedSetters);
+	int i, l;
 
 #ifdef AKILLMAILTO
 	if (
@@ -654,6 +761,35 @@ void timed_akill_queue(char *)
 		ops_email.to = OPSMAILTO;
 		ops_email.from = buf1;
 		ops_email.subject = "Kline log";
+		if (se)
+		{
+			ops_email.body.add("");
+			ops_email.body.add("Excluded Setters/Remover Activity: \n");
+			ops_email.body.add("Nick               Ak+     Ig+     Ah+"
+					   "     ?      Ak-     Ig-      Ah-  --\n");
+			for(; se; se = LIST_NEXT(se, dm_lst))
+			{
+				if (!se->nick) {
+					continue;
+				}
+				l = sprintf(buf2,      "%-18.20s", se->nick);
+
+				for(i = 0 ; i < NUM_AKTYPE_INDICES; i++) {
+					l += sprintf(buf2 + l, " %-7d", se->count_adds[i]);
+
+					se->count_adds[i] = 0;
+				}
+
+				for(i = 0 ; i < NUM_AKTYPE_INDICES; i++) {
+					l += sprintf(buf2 + l, " %-7d", se->count_removes[i]);
+					se->count_removes[i] = 0;
+				}
+				
+				ops_email.body.add(buf2);
+				ops_email.body.add("\n");
+			}
+			ops_email.body.add("\n");
+		}
 		if (ops_enforce_buf.length() > 0) {
 			ops_email.body.add("");
 			ops_email.body.add(kline_enforce_buf.get_string());
@@ -768,25 +904,25 @@ int removeAkill(char *from, char *mask)
  * \param type Type of entry removal is being attempted of (such as #A_AKILL)
  * \param restrict [UNUSED]
  */
-int removeAkillType(char *from, char *mask, int type, int restrict)
+int removeAkillType(char *from, char *given_mask, int type, int restrict)
 {
 	int i, j;
 	struct akill *ak;
-	char *nick, *user = NULL, *host = NULL, *oldmask;
+	char *nick, *user = NULL, *host = NULL, *tmpmask;
 	time_t temp;
 
-	oldmask = strdup(mask);
+	tmpmask = strdup(given_mask);
 
-	j = strlen(mask);
-	nick = mask;
+	j = strlen(tmpmask);
+	nick = tmpmask;
 	for (i = 0; i < j; i++) {
-		if (mask[i] == '!') {
-			mask[i] = 0;
-			user = &(mask[i + 1]);
+		if (tmpmask[i] == '!') {
+			tmpmask[i] = 0;
+			user = &(tmpmask[i + 1]);
 		}
-		if (mask[i] == '@') {
-			mask[i] = 0;
-			host = &(mask[i + 1]);
+		if (tmpmask[i] == '@') {
+			tmpmask[i] = 0;
+			host = &(tmpmask[i + 1]);
 		}
 	}
 
@@ -808,11 +944,11 @@ int removeAkillType(char *from, char *mask, int type, int restrict)
 
 
 				sSend(":%s GLOBOPS :%s is removing %s %s.", OperServ, from,
-					  aktype_str(ak->type, 0), oldmask);
+					  aktype_str(ak->type, 0), given_mask);
 				temp = time(NULL);
 
 #if defined(AKILLMAILTO) || defined(OPSMAILTO)
-				queueakill(oldmask, from, NULL, NULL, temp, ak->type,
+				queueakill(given_mask, from, NULL, NULL, temp, ak->type,
 						   ak->id, 0);
 #endif
 				if (ak->prev)
@@ -833,18 +969,22 @@ int removeAkillType(char *from, char *mask, int type, int restrict)
         */
 				if (strcmp(from, OperServ)) {
 					sSend(":%s NOTICE %s :%s %s removed", OperServ, from,
-						  aktype_str(ak->type, 0), oldmask);
+						  aktype_str(ak->type, 0), given_mask);
 					cancel_timer(ak->tid);	/* If OperServ is removing the kill, timer is up&shouldn't be canceled */
 				}
 				FREE(ak);
-				FREE(oldmask);
+				FREE(tmpmask);
 				return 0;
 			}
 	}
+
+	/* No notices to self */
 	if (strcmp(from, OperServ)) {
 		sSend(":%s NOTICE %s :%s %s not found", OperServ, from,
-			  aktype_str(type, 2), oldmask);
+			  aktype_str(type, 2), given_mask);
 	}
+
+	FREE(tmpmask);
 	return -1;
 }
 
