@@ -3,7 +3,7 @@
  * \author James Hess <mysidia-snn@flame.org>
  * \date 2000, 2001
  *
- * $Id: passwd.c,v 1.2 2004/07/07 05:17:01 Mysid Exp $
+ * $Id: passwd.c,v 1.3 2005/01/07 03:58:10 echostar Exp $
  *
  * Services routines for managing and encoding passwords
  *
@@ -127,7 +127,7 @@ unsigned char *toBase64(const unsigned char *stream, size_t left)
 
 	*p++ = '\0';
 
-	#undef X_OUTPUT_MAP_SZ 64
+	//#define X_OUTPUT_MAP_SZ 64
 	return (unsigned char *)s;
 }
 
